import 'package:bokain_models/src/model_base.dart';

class Day extends ModelBase
{
  static const int startHour = 7;
  static const int startMinute = 0;
  static const int endHour = 19;
  static const int endMinute = 0;

  Day(DateTime start_time) : super()
  {
    DateTime startTime = new DateTime(
        start_time.year,
        start_time.month,
        start_time.day,
        startHour,
        startMinute);

    DateTime endTime = new DateTime(
        start_time.year,
        start_time.month,
        start_time.day,
        endHour,
        endMinute);

    DateTime iTime = startTime;
    while (iTime.isBefore(endTime))
    {
      increments.add(new Increment(iTime));
      iTime = iTime.add(Increment.duration);
    }
  }

  Day.parse(Map<String, dynamic> data) : super.parse(data);

  @override
  void parseData(Map<String, dynamic> data)
  {
    increments.clear();
    userId = data["userId"];
    Iterable<Map<String, String>> activeIncrements = data["activeIncrements"];
    for (Map<String, dynamic> incrementData in activeIncrements)
    {
      DateTime startTime = DateTime.parse(incrementData["startTime"]);
      Increment i = new Increment(startTime);
      i.open = incrementData["open"] as bool;
      i.bookingId = incrementData["bookingId"];
      increments.add(i);
    }
  }

  @override
  Map<String, dynamic> get properties
  {
    Map<String, dynamic> data = new Map();
    data["userId"] = userId;

    Iterable<Increment> activeIncrements = increments.where((i) => i.open || i.bookingId != null);
    data["activeIncrements"] = activeIncrements.map((i) => i.properties);
    return data;
  }

  bool isSameDateAs(DateTime dt)
  {
    DateTime start = increments.first.startTime;
    return (start.year == dt.year && start.month == dt.month && start.day == dt.day);
  }

  String userId;
  List<Increment> increments = new List();
}

class Increment
{
  static const Duration duration = const Duration(minutes: 10);

  Increment(this._startTime)
  {
    _endTime = _startTime.add(duration);
  }

  Map<String, dynamic> get properties
  {
    Map<String, dynamic> data = new Map();
    data["startTime"] = _startTime.toIso8601String();
    data["endTime"] = _endTime.toIso8601String();
    data["bookingId"] = bookingId;
    data["open"] = open;
    return data;
  }

  DateTime get startTime => _startTime;
  DateTime get endTime => _endTime;

  final DateTime _startTime;
  DateTime _endTime;

  bool open = false;
  String bookingId;
}